
// Rotate the matrix CLOCKWISE

//naive implementation: move the elements of the matrix directly to their destinations
//this will cause unaligned memory accessed which - as we will see - should be avoided on the GPU

__kernel void MatrixRotNaive(__global const float* M, __global float* MR, uint SizeX, uint SizeY)
{
	int2 GID;
	GID.x = get_global_id(0);
	GID.y = get_global_id(1);

	if (GID.x < SizeX && GID.x >= 0
		&& GID.y < SizeY && GID.y >= 0){

		MR[GID.x * SizeY + (SizeY - GID.y - 1)] = M[GID.y * SizeX + GID.x];
	}
}

//this kernel does the same thing, however, the local memory is used to
//transform a small chunk of the matrix locally
//then write it back after synchronization in a coalesced access pattern

__kernel void MatrixRotOptimized(__global const float* M, __global float* MR, uint SizeX, uint SizeY,
							__local float* block)
{
	int2 GID;
	GID.x = get_global_id(0);
	GID.y = get_global_id(1);

	int2 LID;
	LID.x = get_local_id(0);
	LID.y = get_local_id(1);

	if (GID.x < SizeX && GID.x >= 0
		&& GID.y < SizeY && GID.y >= 0){

		int blockindex = LID.y * get_local_size(0) + LID.x;
		block[blockindex] = M[GID.y * SizeX + GID.x];

		barrier(CLK_LOCAL_MEM_FENCE);

		int2 newLID;
		newLID.x = blockindex / get_local_size(1);
		newLID.y = blockindex % get_local_size(1);

		int2 newGID = GID - LID + newLID;

		if (newGID.x < SizeX && newGID.x >= 0
			&& newGID.y < SizeY && newGID.y >= 0){
			MR[newGID.x * SizeY + (SizeY - newGID.y - 1)] = block[newLID.y * get_local_size(0) + newLID.x];
		}
	}
}
 